@@ -24,7 +24,7 @@ License along with TPOT. If not, see <http://www.gnu.org/licenses/>.
 """
 
 from tpot import TPOTClassifier, TPOTRegressor
-from tpot.base import TPOTBase
+from tpot.base import TPOTBase, is_notebook
 from tpot.driver import float_range
 from tpot.gp_types import Output_Array
 from tpot.gp_deap import mutNodeReplacement, _wrapped_cross_val_score, pick_two_individuals_eligible_for_crossover, cxOnePoint, varOr, initialize_stats_dict
@@ -115,6 +115,8 @@ TPOTSelectPercentile, TPOTSelectPercentile_args = TPOTOperatorClassFactory(
     classifier_config_dict[test_operator_key]
 )
 
+tpot_obj = TPOTClassifier()
+tpot_obj._fit_init()
 
 def test_init_custom_parameters():
     """Assert that the TPOT instantiator stores the TPOT variables properly."""
@@ -142,10 +144,23 @@ def test_init_custom_parameters():
     assert tpot_obj.max_time_mins is None
     assert tpot_obj.warm_start is True
     assert tpot_obj.verbosity == 1
-    assert tpot_obj._optimized_pipeline is None
-    assert tpot_obj.fitted_pipeline_ is None
-    assert not (tpot_obj._pset is None)
-    assert not (tpot_obj._toolbox is None)
+
+    tpot_obj._fit_init()
+
+    assert tpot_obj._pop == []
+    assert tpot_obj._pareto_front == None
+    assert tpot_obj._last_optimized_pareto_front == None
+    assert tpot_obj._last_optimized_pareto_front_n_gens == 0
+    assert tpot_obj._optimized_pipeline == None
+    assert tpot_obj._optimized_pipeline_score == None
+    assert tpot_obj.fitted_pipeline_ == None
+    assert tpot_obj._exported_pipeline_text == ""
+
+
+def test_is_notebook():
+    """Assert that isnotebook function works as expected."""
+    ret = is_notebook()
+    assert not ret
 
 
 def test_init_default_scoring():
@@ -161,6 +176,7 @@ def test_init_default_scoring_2():
     """Assert that TPOT intitializes with a valid customized metric function."""
     with warnings.catch_warnings(record=True) as w:
         tpot_obj = TPOTClassifier(scoring=balanced_accuracy)
+        tpot_obj._fit_init()
     assert len(w) == 1 # deap 1.2.2 warning message made this unit test failed
     assert issubclass(w[-1].category, DeprecationWarning) # deap 1.2.2 warning message made this unit test failed
     assert "This scoring type was deprecated" in str(w[-1].message) # deap 1.2.2 warning message made this unit test failed
@@ -171,6 +187,7 @@ def test_init_default_scoring_3():
     """Assert that TPOT intitializes with a valid _BaseScorer."""
     with warnings.catch_warnings(record=True) as w:
         tpot_obj = TPOTClassifier(scoring=make_scorer(balanced_accuracy))
+        tpot_obj._fit_init()
     assert len(w) == 0 # deap 1.2.2 warning message made this unit test failed
     assert tpot_obj.scoring_function == 'balanced_accuracy'
 
@@ -182,6 +199,7 @@ def test_init_default_scoring_4():
 
     with warnings.catch_warnings(record=True) as w:
         tpot_obj = TPOTClassifier(scoring=my_scorer)
+        tpot_obj._fit_init()
     assert len(w) == 0 # deap 1.2.2 warning message made this unit test failed
     assert tpot_obj.scoring_function == 'my_scorer'
 
@@ -190,6 +208,7 @@ def test_init_default_scoring_5():
     """Assert that TPOT intitializes with a valid sklearn metric function roc_auc_score."""
     with warnings.catch_warnings(record=True) as w:
         tpot_obj = TPOTClassifier(scoring=roc_auc_score)
+        tpot_obj._fit_init()
     assert len(w) == 1
     assert issubclass(w[-1].category, DeprecationWarning)
     assert "This scoring type was deprecated" in str(w[-1].message)
@@ -202,6 +221,7 @@ def test_init_default_scoring_6():
         return roc_auc_score(y_true, y_pred)
     with warnings.catch_warnings(record=True) as w:
         tpot_obj = TPOTClassifier(scoring=my_scorer)
+        tpot_obj._fit_init()
     assert len(w) == 1
     assert issubclass(w[-1].category, DeprecationWarning)
     assert "This scoring type was deprecated" in str(w[-1].message)
@@ -212,9 +232,10 @@ def test_init_default_scoring_6():
 def test_invalid_score_warning():
     """Assert that the TPOT intitializes raises a ValueError when the scoring metrics is not available in SCORERS."""
     # Mis-spelled scorer
-    assert_raises(ValueError, TPOTClassifier, scoring='balanced_accuray')
+    tpot_obj = TPOTClassifier(scoring='balanced_accuray')
+    assert_raises(ValueError, tpot_obj._fit_init)
     # Correctly spelled
-    TPOTClassifier(scoring='balanced_accuracy')
+    tpot_obj = TPOTClassifier(scoring='balanced_accuracy')
 
 
 def test_invalid_dataset_warning():
@@ -226,6 +247,7 @@ def test_invalid_dataset_warning():
         generations=1,
         verbosity=0
     )
+    tpot_obj._fit_init()
     # common mistake in target
     bad_training_target = training_target.reshape((1, len(training_target)))
     assert_raises(ValueError, tpot_obj.fit, training_features, bad_training_target)
@@ -234,7 +256,8 @@ def test_invalid_dataset_warning():
 def test_invalid_subsample_ratio_warning():
     """Assert that the TPOT intitializes raises a ValueError when subsample ratio is not in the range (0.0, 1.0]."""
     # Invalid ratio
-    assert_raises(ValueError, TPOTClassifier, subsample=0.0)
+    tpot_obj = TPOTClassifier(subsample=0.0)
+    assert_raises(ValueError, tpot_obj._fit_init)
     # Valid ratio
     TPOTClassifier(subsample=0.1)
 
@@ -242,7 +265,8 @@ def test_invalid_subsample_ratio_warning():
 def test_invalid_mut_rate_plus_xo_rate():
     """Assert that the TPOT intitializes raises a ValueError when the sum of crossover and mutation probabilities is large than 1."""
     # Invalid ratio
-    assert_raises(ValueError, TPOTClassifier, mutation_rate=0.8, crossover_rate=0.8)
+    tpot_obj = TPOTClassifier(mutation_rate=0.8, crossover_rate=0.8)
+    assert_raises(ValueError, tpot_obj._fit_init)
     # Valid ratio
     TPOTClassifier(mutation_rate=0.8, crossover_rate=0.1)
 
@@ -250,7 +274,7 @@ def test_invalid_mut_rate_plus_xo_rate():
 def test_init_max_time_mins():
     """Assert that the TPOT init stores max run time and sets generations to 1000000."""
     tpot_obj = TPOTClassifier(max_time_mins=30, generations=1000)
-
+    tpot_obj._fit_init()
     assert tpot_obj.generations == 1000000
     assert tpot_obj.max_time_mins == 30
 
@@ -261,12 +285,15 @@ def test_init_n_jobs():
     assert tpot_obj.n_jobs == 2
 
     tpot_obj = TPOTClassifier(n_jobs=-1)
-    assert tpot_obj.n_jobs == cpu_count()
+    assert tpot_obj.n_jobs == -1
+    tpot_obj._fit_init()
+    assert tpot_obj._n_jobs == cpu_count()
 
 
 def test_timeout():
     """Assert that _wrapped_cross_val_score return Timeout in a time limit."""
     tpot_obj = TPOTRegressor(scoring='neg_mean_squared_error')
+    tpot_obj._fit_init()
     # a complex pipeline for the test
     pipeline_string = (
         "ExtraTreesRegressor("
@@ -295,7 +322,7 @@ def test_timeout():
 
 def test_invalid_pipeline():
     """Assert that _wrapped_cross_val_score return -float(\'inf\') with a invalid_pipeline"""
-    tpot_obj = TPOTClassifier()
+
     # a invalid pipeline
     # Dual or primal formulation. Dual formulation is only implemented for l2 penalty.
     pipeline_string = (
@@ -342,15 +369,14 @@ def test_get_params():
     initializer = inspect.getargspec(TPOTBase.__init__)
     default_kwargs = dict(zip(initializer.args[1:], initializer.defaults))
     default_kwargs.update(kwargs)
-    # update to dictionary instead of input string
-    default_kwargs.update({'config_dict': classifier_config_dict_light})
-    assert tpot_obj.get_params()['config_dict'] == default_kwargs['config_dict']
+
+    assert tpot_obj.get_params()['config_dict'] == 'TPOT light'
     assert tpot_obj.get_params() == default_kwargs
 
 
 def test_set_params():
     """Assert that set_params returns a reference to the TPOT instance."""
-    tpot_obj = TPOTClassifier()
+
     assert tpot_obj.set_params() is tpot_obj
 
 
@@ -370,22 +396,28 @@ def test_TPOTBase():
 def test_conf_dict():
     """Assert that TPOT uses the pre-configured dictionary of operators when config_dict is 'TPOT light' or 'TPOT MDR'."""
     tpot_obj = TPOTClassifier(config_dict='TPOT light')
-    assert tpot_obj.config_dict == classifier_config_dict_light
+    tpot_obj._fit_init()
+    assert tpot_obj._config_dict == classifier_config_dict_light
 
     tpot_obj = TPOTClassifier(config_dict='TPOT MDR')
-    assert tpot_obj.config_dict == tpot_mdr_classifier_config_dict
+    tpot_obj._fit_init()
+    assert tpot_obj._config_dict == tpot_mdr_classifier_config_dict
 
     tpot_obj = TPOTClassifier(config_dict='TPOT sparse')
-    assert tpot_obj.config_dict == classifier_config_sparse
+    tpot_obj._fit_init()
+    assert tpot_obj._config_dict == classifier_config_sparse
 
     tpot_obj = TPOTRegressor(config_dict='TPOT light')
-    assert tpot_obj.config_dict == regressor_config_dict_light
+    tpot_obj._fit_init()
+    assert tpot_obj._config_dict == regressor_config_dict_light
 
     tpot_obj = TPOTRegressor(config_dict='TPOT MDR')
-    assert tpot_obj.config_dict == tpot_mdr_regressor_config_dict
+    tpot_obj._fit_init()
+    assert tpot_obj._config_dict == tpot_mdr_regressor_config_dict
 
     tpot_obj = TPOTRegressor(config_dict='TPOT sparse')
-    assert tpot_obj.config_dict == regressor_config_sparse
+    tpot_obj._fit_init()
+    assert tpot_obj._config_dict == regressor_config_sparse
 
 
 def test_conf_dict_2():
@@ -397,6 +429,7 @@ def test_conf_dict_2():
 def test_conf_dict_3():
     """Assert that TPOT uses a custom dictionary of operators when config_dict is the path of Python dictionary."""
     tpot_obj = TPOTRegressor(config_dict='tests/test_config.py')
+    tpot_obj._fit_init()
     tested_config_dict = {
         'sklearn.naive_bayes.GaussianNB': {
         },
@@ -411,8 +444,10 @@ def test_conf_dict_3():
             'fit_prior': [True, False]
         }
     }
-    assert isinstance(tpot_obj.config_dict, dict)
-    assert tpot_obj.config_dict == tested_config_dict
+
+    assert isinstance(tpot_obj.config_dict, str)
+    assert isinstance(tpot_obj._config_dict, dict)
+    assert tpot_obj._config_dict == tested_config_dict
 
 
 def test_read_config_file():
@@ -437,8 +472,10 @@ def test_read_config_file_3():
 def test_random_ind():
     """Assert that the TPOTClassifier can generate the same pipeline with same random seed."""
     tpot_obj = TPOTClassifier(random_state=43)
+    tpot_obj._fit_init()
     pipeline1 = str(tpot_obj._toolbox.individual())
     tpot_obj = TPOTClassifier(random_state=43)
+    tpot_obj._fit_init()
     pipeline2 = str(tpot_obj._toolbox.individual())
     assert pipeline1 == pipeline2
 
@@ -446,8 +483,10 @@ def test_random_ind():
 def test_random_ind_2():
     """Assert that the TPOTRegressor can generate the same pipeline with same random seed."""
     tpot_obj = TPOTRegressor(random_state=43)
+    tpot_obj._fit_init()
     pipeline1 = str(tpot_obj._toolbox.individual())
     tpot_obj = TPOTRegressor(random_state=43)
+    tpot_obj._fit_init()
     pipeline2 = str(tpot_obj._toolbox.individual())
 
     assert pipeline1 == pipeline2
@@ -456,13 +495,14 @@ def test_random_ind_2():
 def test_score():
     """Assert that the TPOT score function raises a RuntimeError when no optimized pipeline exists."""
     tpot_obj = TPOTClassifier()
-
+    tpot_obj._fit_init()
     assert_raises(RuntimeError, tpot_obj.score, testing_features, testing_target)
 
 
 def test_score_2():
     """Assert that the TPOTClassifier score function outputs a known score for a fixed pipeline."""
     tpot_obj = TPOTClassifier(random_state=34)
+    tpot_obj._fit_init()
     known_score = 0.977777777778  # Assumes use of the TPOT accuracy function
 
     # Create a pipeline with a known score
@@ -486,6 +526,7 @@ def test_score_2():
 def test_score_3():
     """Assert that the TPOTRegressor score function outputs a known score for a fixed pipeline."""
     tpot_obj = TPOTRegressor(scoring='neg_mean_squared_error', random_state=72)
+    tpot_obj._fit_init()
     known_score = -12.1791953611
 
     # Reify pipeline with known score
@@ -513,7 +554,7 @@ def test_score_3():
 def test_sample_weight_func():
     """Assert that the TPOTRegressor score function outputs a known score for a fixed pipeline with sample weights."""
     tpot_obj = TPOTRegressor(scoring='neg_mean_squared_error')
-
+    tpot_obj._fit_init()
     # Reify pipeline with known scor
     pipeline_string = (
         "ExtraTreesRegressor("
@@ -580,13 +621,14 @@ def test_fit_GroupKFold():
 def test_predict():
     """Assert that the TPOT predict function raises a RuntimeError when no optimized pipeline exists."""
     tpot_obj = TPOTClassifier()
-
+    tpot_obj._fit_init()
     assert_raises(RuntimeError, tpot_obj.predict, testing_features)
 
 
 def test_predict_2():
     """Assert that the TPOT predict function returns a numpy matrix of shape (num_testing_rows,)."""
     tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
     pipeline_string = (
         'DecisionTreeClassifier('
         'input_matrix, '
@@ -604,9 +646,33 @@ def test_predict_2():
     assert result.shape == (testing_features.shape[0],)
 
 
+def test_predict_3():
+    """Assert that the TPOT predict function works on dataset with nan"""
+    tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
+
+    pipeline_string = (
+        'DecisionTreeClassifier('
+        'input_matrix, '
+        'DecisionTreeClassifier__criterion=gini, '
+        'DecisionTreeClassifier__max_depth=8, '
+        'DecisionTreeClassifier__min_samples_leaf=5, '
+        'DecisionTreeClassifier__min_samples_split=5'
+        ')'
+    )
+    tpot_obj._optimized_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    tpot_obj.fitted_pipeline_ = tpot_obj._toolbox.compile(expr=tpot_obj._optimized_pipeline)
+    tpot_obj.fitted_pipeline_.fit(training_features, training_target)
+    result = tpot_obj.predict(features_with_nan)
+
+    assert result.shape == (features_with_nan.shape[0],)
+
+
 def test_predict_proba():
     """Assert that the TPOT predict_proba function returns a numpy matrix of shape (num_testing_rows, num_testing_target)."""
     tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
+
     pipeline_string = (
         'DecisionTreeClassifier('
         'input_matrix, '
@@ -628,6 +694,7 @@ def test_predict_proba():
 def test_predict_proba_2():
     """Assert that the TPOT predict_proba function returns a numpy matrix filled with probabilities (float)."""
     tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
     pipeline_string = (
         'DecisionTreeClassifier('
         'input_matrix, '
@@ -651,6 +718,7 @@ def test_predict_proba_2():
 def test_predict_proba_3():
     """Assert that the TPOT predict_proba function raises a RuntimeError when no optimized pipeline exists."""
     tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
 
     assert_raises(RuntimeError, tpot_obj.predict_proba, testing_features)
 
@@ -658,6 +726,7 @@ def test_predict_proba_3():
 def test_predict_proba_4():
     """Assert that the TPOT predict_proba function raises a RuntimeError when the optimized pipeline do not have the predict_proba() function"""
     tpot_obj = TPOTRegressor()
+    tpot_obj._fit_init()
     pipeline_string = (
         "ExtraTreesRegressor(input_matrix, "
         "ExtraTreesRegressor__bootstrap=True, ExtraTreesRegressor__max_features=0.5,"
@@ -671,6 +740,28 @@ def test_predict_proba_4():
     assert_raises(RuntimeError, tpot_obj.predict_proba, testing_features)
 
 
+def test_predict_proba_5():
+    """Assert that the TPOT predict_proba function works on dataset with nan."""
+    tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
+    pipeline_string = (
+        'DecisionTreeClassifier('
+        'input_matrix, '
+        'DecisionTreeClassifier__criterion=gini, '
+        'DecisionTreeClassifier__max_depth=8, '
+        'DecisionTreeClassifier__min_samples_leaf=5, '
+        'DecisionTreeClassifier__min_samples_split=5)'
+    )
+    tpot_obj._optimized_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
+    tpot_obj.fitted_pipeline_ = tpot_obj._toolbox.compile(expr=tpot_obj._optimized_pipeline)
+    tpot_obj.fitted_pipeline_.fit(training_features, training_target)
+
+    result = tpot_obj.predict_proba(features_with_nan)
+    num_labels = np.amax(training_target) + 1
+
+    assert result.shape == (features_with_nan.shape[0], num_labels)
+
+
 def test_warm_start():
     """Assert that the TPOT warm_start flag stores the pop and pareto_front from the first run."""
     tpot_obj = TPOTClassifier(
@@ -691,6 +782,8 @@ def test_warm_start():
     tpot_obj.fit(pretest_X, pretest_y)
 
     assert tpot_obj._pop == first_pop
+    assert tpot_obj._pop == first_pop
+    assert tpot_obj._pop == first_pop
 
 
 def test_fit():
@@ -751,6 +844,7 @@ def test_fit_4():
         max_time_mins=2/60.,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
     assert tpot_obj.generations == 1000000
 
     # reset generations to 20 just in case that the failed test may take too much time
@@ -1117,6 +1211,7 @@ def test_summary_of_best_pipeline():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     assert_raises(RuntimeError, tpot_obj._summary_of_best_pipeline, features=training_features, target=training_target)
 
@@ -1128,7 +1223,7 @@ def test_set_param_recursive():
         'DecisionTreeClassifier__criterion=gini, DecisionTreeClassifier__max_depth=8, '
         'DecisionTreeClassifier__min_samples_leaf=5, DecisionTreeClassifier__min_samples_split=5)'
     )
-    tpot_obj = TPOTClassifier()
+
     deap_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
     sklearn_pipeline = tpot_obj._toolbox.compile(expr=deap_pipeline)
     tpot_obj._set_param_recursive(sklearn_pipeline.steps, 'random_state', 42)
@@ -1147,6 +1242,7 @@ def test_set_param_recursive_2():
         'DecisionTreeRegressor__min_samples_leaf=5, DecisionTreeRegressor__min_samples_split=5)'
     )
     tpot_obj = TPOTRegressor()
+    tpot_obj._fit_init()
     deap_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
     sklearn_pipeline = tpot_obj._toolbox.compile(expr=deap_pipeline)
     tpot_obj._set_param_recursive(sklearn_pipeline.steps, 'random_state', 42)
@@ -1165,7 +1261,7 @@ def test_set_param_recursive_3():
         'DecisionTreeClassifier__criterion=gini, DecisionTreeClassifier__max_depth=8, '
         'DecisionTreeClassifier__min_samples_leaf=5, DecisionTreeClassifier__min_samples_split=5)'
     )
-    tpot_obj = TPOTClassifier()
+
     deap_pipeline = creator.Individual.from_string(pipeline_string, tpot_obj._pset)
     sklearn_pipeline = tpot_obj._toolbox.compile(expr=deap_pipeline)
     tpot_obj._set_param_recursive(sklearn_pipeline.steps, 'random_state', 42)
@@ -1224,6 +1320,7 @@ def test_evaluate_individuals():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     tpot_obj._pbar = tqdm(total=1, disable=True)
     pop = tpot_obj._toolbox.population(n=10)
@@ -1253,6 +1350,7 @@ def test_evaluate_individuals_2():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     tpot_obj._pbar = tqdm(total=1, disable=True)
     pop = tpot_obj._toolbox.population(n=10)
@@ -1281,6 +1379,7 @@ def test_update_pbar():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
@@ -1299,6 +1398,7 @@ def test_update_val():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
@@ -1320,6 +1420,7 @@ def test_preprocess_individuals():
         random_state=42,
         verbosity=0
     )
+    tpot_obj._fit_init()
 
     pipeline_string_1 = (
         'LogisticRegression(PolynomialFeatures'
@@ -1365,6 +1466,7 @@ def test_preprocess_individuals_2():
         random_state=42,
         verbosity=0
     )
+    tpot_obj._fit_init()
 
     # pipeline with two PolynomialFeatures operator
     pipeline_string_1 = (
@@ -1414,6 +1516,7 @@ def test_preprocess_individuals_3():
         max_time_mins=5,
         verbosity=0
     )
+    tpot_obj._fit_init()
 
     pipeline_string_1 = (
         'LogisticRegression(PolynomialFeatures'
@@ -1440,6 +1543,7 @@ def test_preprocess_individuals_3():
 
     with closing(StringIO()) as our_file:
         tpot_obj._file=our_file
+        tpot_obj._lambda=4
         tpot_obj._pbar = tqdm(total=2, disable=False, file=our_file)
         tpot_obj._pbar.n = 2
         operator_counts, eval_individuals_str, sklearn_pipeline_list, stats_dicts = \
@@ -1447,6 +1551,85 @@ def test_preprocess_individuals_3():
         assert tpot_obj._pbar.total == 6
 
 
+def test_check_dataset():
+    """Assert that the check_dataset function returns feature and target as expected."""
+    tpot_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT light'
+    )
+
+    ret_features, ret_target = tpot_obj._check_dataset(training_features, training_target)
+    assert np.allclose(ret_features, training_features)
+    assert np.allclose(ret_target, training_target)
+
+
+def test_check_dataset_2():
+    """Assert that the check_dataset function raise ValueError when sample_weight can not be converted to float array"""
+    tpot_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT light'
+    )
+    test_sample_weight = list(range(1, len(training_target)+1))
+    ret_features, ret_target = tpot_obj._check_dataset(training_features, training_target, test_sample_weight)
+    test_sample_weight[0] = 'opps'
+
+    assert_raises(ValueError, tpot_obj._check_dataset, training_features, training_target, test_sample_weight)
+
+
+def test_check_dataset_3():
+    """Assert that the check_dataset function raise ValueError when sample_weight has NaN"""
+    tpot_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT light'
+    )
+    test_sample_weight = list(range(1, len(training_target)+1))
+    ret_features, ret_target = tpot_obj._check_dataset(training_features, training_target, test_sample_weight)
+    test_sample_weight[0] = np.nan
+
+    assert_raises(ValueError, tpot_obj._check_dataset, training_features, training_target, test_sample_weight)
+
+
+def test_check_dataset_4():
+    """Assert that the check_dataset function raise ValueError when sample_weight has a length different length"""
+    tpot_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT light'
+    )
+    test_sample_weight = list(range(1, len(training_target)))
+    assert_raises(ValueError, tpot_obj._check_dataset, training_features, training_target, test_sample_weight)
+
+
+def test_check_dataset_5():
+    """Assert that the check_dataset function returns feature and target as expected."""
+    tpot_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT light'
+    )
+
+    ret_features = tpot_obj._check_dataset(training_features, target=None)
+    assert np.allclose(ret_features, training_features)
+
+
 def test_imputer():
     """Assert that the TPOT fit function will not raise a ValueError in a dataset where NaNs are present."""
     tpot_obj = TPOTClassifier(
@@ -1489,6 +1672,7 @@ def test_imputer_3():
         verbosity=2,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     with captured_output() as (out, err):
         imputed_features = tpot_obj._impute_values(features_with_nan)
@@ -1624,7 +1808,7 @@ def test_tpot_operator_factory_class():
 
 def test_PolynomialFeatures_exception():
     """Assert that TPOT allows only one PolynomialFeatures operator in a pipeline."""
-    tpot_obj = TPOTClassifier()
+
     tpot_obj._pbar = tqdm(total=1, disable=True)
     # pipeline with one PolynomialFeatures operator
     pipeline_string_1 = (
@@ -1659,7 +1843,7 @@ def test_PolynomialFeatures_exception():
 
 def test_pick_two_individuals_eligible_for_crossover():
     """Assert that pick_two_individuals_eligible_for_crossover() picks the correct pair of nodes to perform crossover with"""
-    tpot_obj = TPOTClassifier()
+
     ind1 = creator.Individual.from_string(
         'BernoulliNB(input_matrix, BernoulliNB__alpha=1.0, BernoulliNB__fit_prior=True)',
         tpot_obj._pset
@@ -1695,7 +1879,7 @@ def test_pick_two_individuals_eligible_for_crossover():
 
 def test_pick_two_individuals_eligible_for_crossover_bad():
     """Assert that pick_two_individuals_eligible_for_crossover() returns the right output when no pair is eligible"""
-    tpot_obj = TPOTClassifier()
+
     ind1 = creator.Individual.from_string(
         'BernoulliNB(input_matrix, BernoulliNB__alpha=1.0, BernoulliNB__fit_prior=True)',
         tpot_obj._pset
@@ -1724,7 +1908,7 @@ def test_pick_two_individuals_eligible_for_crossover_bad():
 
 def test_mate_operator():
     """Assert that self._mate_operator returns offsprings as expected."""
-    tpot_obj = TPOTClassifier()
+
     ind1 = creator.Individual.from_string(
         'KNeighborsClassifier('
         'BernoulliNB(input_matrix, BernoulliNB__alpha=10.0, BernoulliNB__fit_prior=False),'
@@ -1775,7 +1959,7 @@ def test_mate_operator():
 
 def test_cxOnePoint():
     """Assert that cxOnePoint() returns the correct type of node between two fixed pipelines."""
-    tpot_obj = TPOTClassifier()
+
     ind1 = creator.Individual.from_string(
         'KNeighborsClassifier('
         'BernoulliNB(input_matrix, BernoulliNB__alpha=10.0, BernoulliNB__fit_prior=False),'
@@ -1805,7 +1989,7 @@ def test_cxOnePoint():
 
 def test_mutNodeReplacement():
     """Assert that mutNodeReplacement() returns the correct type of mutation node in a fixed pipeline."""
-    tpot_obj = TPOTClassifier()
+
     pipeline_string = (
         'LogisticRegression(PolynomialFeatures'
         '(input_matrix, PolynomialFeatures__degree=2, PolynomialFeatures__include_bias=False, '
@@ -1837,6 +2021,7 @@ def test_mutNodeReplacement():
 def test_mutNodeReplacement_2():
     """Assert that mutNodeReplacement() returns the correct type of mutation node in a complex pipeline."""
     tpot_obj = TPOTClassifier()
+    tpot_obj._fit_init()
     # a pipeline with 4 operators
     pipeline_string = (
         "LogisticRegression("
@@ -1880,6 +2065,7 @@ def test_varOr():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     tpot_obj._pbar = tqdm(total=1, disable=True)
     pop = tpot_obj._toolbox.population(n=5)
@@ -1902,6 +2088,7 @@ def test_varOr_2():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     tpot_obj._pbar = tqdm(total=1, disable=True)
     pop = tpot_obj._toolbox.population(n=5)
@@ -1924,6 +2111,7 @@ def test_varOr_3():
         verbosity=0,
         config_dict='TPOT light'
     )
+    tpot_obj._fit_init()
 
     tpot_obj._pbar = tqdm(total=1, disable=True)
     pop = tpot_obj._toolbox.population(n=5)
@@ -1944,7 +2132,7 @@ def test_operator_type():
 
 def test_gen():
     """Assert that TPOT's gen_grow_safe function returns a pipeline of expected structure."""
-    tpot_obj = TPOTClassifier()
+
 
     pipeline = tpot_obj._gen_grow_safe(tpot_obj._pset, 1, 3)
 
@@ -1957,7 +2145,7 @@ def test_clean_pipeline_string():
 
     with_prefix = 'BernoulliNB(input_matrix, BernoulliNB__alpha=1.0, BernoulliNB__fit_prior=True)'
     without_prefix = 'BernoulliNB(input_matrix, alpha=1.0, fit_prior=True)'
-    tpot_obj = TPOTClassifier()
+
     ind1 = creator.Individual.from_string(with_prefix, tpot_obj._pset)
 
     pretty_string = tpot_obj.clean_pipeline_string(ind1)
